from typing import TYPE_CHECKING, cast

from django.db import models
from django.utils.text import slugify

if TYPE_CHECKING:
    from ...administration import CenterProduct, CenterResource, CenterWaste
    from ...media import AnonymExaminationReport, AnonymHistologyReport
    from ...medical import Endoscope, EndoscopyProcessor
    from ..person.names.first_name import FirstName
    from ..person.names.last_name import LastName


class CenterManager(models.Manager):
    def get_by_natural_key(self, name) -> "Center":
        return cast("Center", self.get(name=name))

    def get_by_center_key(self, center_key: str) -> "Center":
        return cast("Center", self.get(center_key=center_key))


class Center(models.Model):
    objects = CenterManager()
    name = models.CharField(max_length=255)
    center_key = models.CharField(max_length=255, unique=True, blank=True)
    display_name = models.CharField(max_length=255, blank=True, default="")

    first_names = models.ManyToManyField(
        to="FirstName",
        related_name="centers",
    )
    last_names = models.ManyToManyField("LastName", related_name="centers")

    if TYPE_CHECKING:
        from django.db.models.manager import RelatedManager

        first_names = cast(RelatedManager[FirstName], first_names)
        last_names = cast(RelatedManager[LastName], last_names)

        @property
        def center_products(self) -> RelatedManager[CenterProduct]: ...

        @property
        def center_resources(self) -> RelatedManager[CenterResource]: ...

        @property
        def center_wastes(self) -> RelatedManager[CenterWaste]: ...

        @property
        def endoscopy_processors(self) -> RelatedManager[EndoscopyProcessor]: ...

        @property
        def endoscopes(self) -> RelatedManager[Endoscope]: ...

        @property
        def anonymexaminationreport_set(
            self,
        ) -> RelatedManager[AnonymExaminationReport]: ...

        @property
        def anonymhistologyreport_set(
            self,
        ) -> RelatedManager[AnonymHistologyReport]: ...

    @classmethod
    def get_by_name(cls, name):
        return cls.objects.get(name=name)

    @classmethod
    def get_by_center_key(cls, center_key: str):
        return cls.objects.get(center_key=center_key)

    @classmethod
    def resolve_identity(cls, identifier: str):
        return (
            cls.objects.filter(center_key=identifier).first()
            or cls.objects.filter(name=identifier).first()
        )

    def natural_key(self) -> tuple[str]:
        return (self.name,)

    @classmethod
    def build_center_key(cls, value: str, *, exclude_pk: int | None = None) -> str:
        base = slugify(value or "") or "center"
        candidate = base
        suffix = 2
        queryset = cls.objects.all()
        if exclude_pk is not None:
            queryset = queryset.exclude(pk=exclude_pk)
        while queryset.filter(center_key=candidate).exists():
            candidate = f"{base}-{suffix}"
            suffix += 1
        return candidate

    def save(self, *args, **kwargs):
        if self.pk:
            existing_key = (
                type(self)
                .objects.filter(pk=self.pk)
                .values_list("center_key", flat=True)
                .first()
            )
            if existing_key and self.center_key and self.center_key != existing_key:
                raise ValueError("center_key is immutable once assigned")

        if not self.center_key:
            source_value = self.display_name or self.name
            self.center_key = self.build_center_key(
                source_value,
                exclude_pk=self.pk,
            )
        if not self.display_name:
            self.display_name = self.name
        super().save(*args, **kwargs)

    def __str__(self) -> str:
        return str(self.display_name or self.name)

    def get_first_names(self):
        return self.first_names.all()

    def get_last_names(self):
        return self.last_names.all()

    def get_endoscopes(self):
        """
        Returns all Endoscope instances associated with this center.
        """
        return self.endoscopes.all()



from django.db import migrations, models


def deduplicate_image_classification_annotations(apps, schema_editor):
    image_classification_annotation = apps.get_model(
        "endoreg_db", "ImageClassificationAnnotation"
    )
    db_alias = schema_editor.connection.alias

    duplicate_groups = (
        image_classification_annotation.objects.using(db_alias)
        .values("frame_id", "label_id", "information_source_id", "annotator")
        .annotate(row_count=models.Count("id"))
        .filter(row_count__gt=1)
    )

    for group in duplicate_groups.iterator():
        group_filters = {
            "frame_id": group["frame_id"],
            "label_id": group["label_id"],
            "information_source_id": group["information_source_id"],
            "annotator": group["annotator"],
        }
        group_qs = image_classification_annotation.objects.using(db_alias).filter(
            **group_filters
        )
        keep_id = (
            group_qs.order_by("-date_modified", "-id")
            .values_list("id", flat=True)
            .first()
        )
        if keep_id is None:
            continue
        group_qs.exclude(id=keep_id).delete()


class Migration(migrations.Migration):
    atomic = False

    dependencies = [
        ("endoreg_db", "0007_pdfprocessinghistory"),
    ]

    operations = [
        migrations.AddField(
            model_name="imageclassificationannotation",
            name="external_annotation_id",
            field=models.CharField(
                blank=True,
                db_index=True,
                max_length=255,
                null=True,
            ),
        ),
        migrations.RunPython(
            deduplicate_image_classification_annotations,
            reverse_code=migrations.RunPython.noop,
        ),
        migrations.AddConstraint(
            model_name="imageclassificationannotation",
            constraint=models.UniqueConstraint(
                fields=("frame", "label", "information_source", "annotator"),
                name="uniq_frame_label_source_annotator",
            ),
        ),
    ]



from endoreg_db.import_files.report_import_service import ReportImportService as ris


class ReportImportService(ris):
    __doc__ = (
        (ris.__doc__ or "")
        + """
    Wrapper for known import of concurrency safe ReportImportService, 
    here celery will be implemented in the future.
    12.02.2025
    """
    )
    pass



from .file_overview import FileOverviewSerializer
from .sensitive_patient_data import VoPPatientDataSerializer
from .stats import StatsSerializer
from ..hub import (
    TransferJobCreateSerializer,
    TransferJobStatusSerializer,
    UploadCreateResponseSerializer,
    UploadJobStatusSerializer,
)
from .translatable_field_mix_in import TranslatableFieldMixin

__all__ = [
    "FileOverviewSerializer",
    "VoPPatientDataSerializer",
    "StatsSerializer",
    "TransferJobCreateSerializer",
    "TransferJobStatusSerializer",
    "UploadJobStatusSerializer",
    "UploadCreateResponseSerializer",
    "TranslatableFieldMixin",
]



from .correction import (
    VideoProcessingHistoryView,
    VideoApplyMaskView,
    VideoRemoveFramesView,
    VideoCorrectionView,
)

from ..media.video_media import VideoMediaView

from .reimport import VideoReimportView
from .video_stream import VideoStreamView
from .video_examination_viewset import VideoExaminationViewSet
from .video_fps import VideoFpsView
from .video_metadata import VideoMetadataStatsView
from .export_annotated import export_annotated_data


from .segments_crud import (
    video_segments_collection,
    video_segments_by_video,
    video_segment_detail,
    video_segments_stats,
    video_segment_validate,
    video_segments_validate_bulk,
    video_segments_validation_status,
    ensure_segment_annotations_bulk,
    ensure_segment_annotations_for_video,
    ensure_prediction_segment_annotations_bulk,
    ensure_prediction_segment_annotations_for_video,
)
from .ai.label import label_list

__all__ = [
    # Video Correction (Phase 1.1) - Implemented
    "VideoProcessingHistoryView",
    "VideoApplyMaskView",
    "VideoRemoveFramesView",
    "VideoMetadataStatsView",
    "VideoFpsView",
    "VideoMediaView",
    "VideoCorrectionView",
    # Reimport views
    "VideoReimportView",
    # Segmentation views
    "VideoStreamView",
    # Video Examination ViewSet
    "VideoExaminationViewSet",
    "video_segments_collection",
    "video_segments_by_video",
    "video_segment_detail",
    "video_segments_stats",
    "video_segment_validate",
    "video_segments_validate_bulk",
    "video_segments_validation_status",
    "ensure_segment_annotations_bulk",
    "ensure_segment_annotations_for_video",
    "ensure_prediction_segment_annotations_bulk",
    "ensure_prediction_segment_annotations_for_video",
    "label_list",
    "export_annotated_data",
]



import hashlib
import logging
from typing import Any

from django.db import transaction
from lx_dtypes.models.contracts import (
    PdfRedactionRequest,
    PdfRedactionResponse,
    ValidationError,
)
from rest_framework import status
from rest_framework.parsers import FormParser, MultiPartParser
from rest_framework.response import Response
from rest_framework.views import APIView

from endoreg_db.authz.permissions import PolicyPermission
from endoreg_db.models import PdfProcessingHistory, RawPdfFile
from endoreg_db.serializers.pdf.pdf_processing_history import (
    PdfProcessingHistorySerializer,
)
from endoreg_db.services.polling_coordinator import ProcessingLockContext
from endoreg_db.utils.file_operations import sha256_file
from endoreg_db.utils.operation_log import record_operation
from endoreg_db.utils.permissions import EnvironmentAwarePermission
from endoreg_db.utils.storage import ensure_local_file

logger = logging.getLogger(__name__)

MAX_REDACTION_UPLOAD_BYTES = 100 * 1024 * 1024
ALLOWED_PDF_CONTENT_TYPES = {"application/pdf", "application/x-pdf"}
PDF_MAGIC_HEADER = b"%PDF-"


def _state_status_value(state_obj: Any) -> str | None:
    if state_obj is None:
        return None
    status_value = getattr(state_obj, "anonymization_status", None)
    if status_value is None:
        return None
    return str(getattr(status_value, "value", status_value))


def _is_pdf_file(uploaded_file) -> bool:
    try:
        uploaded_file.seek(0)
        header = uploaded_file.read(len(PDF_MAGIC_HEADER))
    finally:
        uploaded_file.seek(0)
    return header == PDF_MAGIC_HEADER


def _sha256_uploaded_file(uploaded_file) -> str:
    digest = hashlib.sha256()
    try:
        uploaded_file.seek(0)
        for chunk in uploaded_file.chunks():
            digest.update(chunk)
    finally:
        uploaded_file.seek(0)
    return digest.hexdigest()


def _sha256_field_file(field_file) -> str | None:
    if not field_file or not getattr(field_file, "name", ""):
        return None
    try:
        with ensure_local_file(field_file) as local_path:
            return sha256_file(local_path)
    except Exception:
        logger.exception("Failed to compute source sha256 for %s", field_file.name)
        return None


class PdfApplyRedactionsView(APIView):
    """
    POST /api/media/pdfs/<pk>/apply-redactions/
    """

    permission_classes = [EnvironmentAwarePermission, PolicyPermission]
    parser_classes = [MultiPartParser, FormParser]

    def post(self, request, pk: int):
        pdf = RawPdfFile.objects.select_related("state").filter(pk=pk).first()
        if pdf is None:
            return Response(
                {"error": f"report with ID {pk} not found."},
                status=status.HTTP_404_NOT_FOUND,
            )

        with ProcessingLockContext(pk, file_type="pdf") as lock:
            if not lock.acquired:
                return Response(
                    {"error": "report is currently locked for processing."},
                    status=status.HTTP_409_CONFLICT,
                )

            try:
                uploaded_pdf = request.FILES.get("file")
                if uploaded_pdf is None:
                    return Response(
                        {"error": "file is required."},
                        status=status.HTTP_400_BAD_REQUEST,
                    )

                file_size = getattr(uploaded_pdf, "size", 0)
                if (
                    isinstance(file_size, int)
                    and file_size > MAX_REDACTION_UPLOAD_BYTES
                ):
                    return Response(
                        {
                            "error": (
                                "uploaded file exceeds maximum size of "
                                f"{MAX_REDACTION_UPLOAD_BYTES} bytes."
                            )
                        },
                        status=status.HTTP_413_REQUEST_ENTITY_TOO_LARGE,
                    )

                content_type = str(getattr(uploaded_pdf, "content_type", "") or "")
                if (
                    content_type
                    and content_type.lower() not in ALLOWED_PDF_CONTENT_TYPES
                ):
                    return Response(
                        {"error": "unsupported media type; expected application/pdf."},
                        status=status.HTTP_415_UNSUPPORTED_MEDIA_TYPE,
                    )

                if not _is_pdf_file(uploaded_pdf):
                    return Response(
                        {"error": "uploaded file is not a valid pdf."},
                        status=status.HTTP_400_BAD_REQUEST,
                    )

                try:
                    payload = PdfRedactionRequest.model_validate(
                        {
                            "source_type": request.data.get("source_type", ""),
                            "redaction_manifest": request.data.get(
                                "redaction_manifest"
                            ),
                            "note": request.data.get("note", ""),
                            "client_source_sha256": request.data.get(
                                "client_source_sha256", ""
                            ),
                        }
                    )
                except ValidationError as exc:
                    return Response(
                        {"error": f"invalid redaction_manifest: {exc}"},
                        status=status.HTTP_400_BAD_REQUEST,
                    )
                source_type = payload.source_type
                note = payload.note
                client_source_sha256 = payload.client_source_sha256
                manifest = payload.redaction_manifest.model_dump(mode="python")

                source_field = (
                    pdf.file
                    if source_type == PdfProcessingHistory.SOURCE_TYPE_RAW
                    else pdf.processed_file
                )
                source_sha256 = _sha256_field_file(source_field)
                if source_sha256 is None:
                    return Response(
                        {
                            "error": "source file is unavailable for the requested source_type."
                        },
                        status=status.HTTP_409_CONFLICT,
                    )

                if client_source_sha256 and client_source_sha256 != source_sha256:
                    return Response(
                        {
                            "error": "source hash mismatch.",
                            "expected_source_sha256": source_sha256,
                        },
                        status=status.HTTP_409_CONFLICT,
                    )

                uploaded_sha256 = _sha256_uploaded_file(uploaded_pdf)
                processed_filename = (
                    f"{pdf.pdf_hash}_redaction_{uploaded_sha256[:12]}.pdf"
                )

                with transaction.atomic():
                    pdf = (
                        RawPdfFile.objects.select_related("state")
                        .select_for_update()
                        .filter(pk=pk)
                        .first()
                    )
                    if pdf is None:
                        return Response(
                            {"error": f"report with ID {pk} not found."},
                            status=status.HTTP_404_NOT_FOUND,
                        )

                    state = pdf.get_or_create_state()
                    status_before = _state_status_value(state) or "not_started"

                    pdf.processed_file.save(
                        processed_filename, uploaded_pdf, save=False
                    )
                    pdf.save(update_fields=["processed_file", "date_modified"])

                    state_update_fields = [
                        "anonymized",
                        "anonymization_validated",
                        "date_modified",
                    ]
                    state.anonymized = True
                    state.anonymization_validated = False
                    # Keep existing value unless missing; a redacted upload is processed output.
                    if not state.sensitive_meta_processed:
                        state.sensitive_meta_processed = True
                        state_update_fields.append("sensitive_meta_processed")
                    state.save(update_fields=state_update_fields)

                    user = getattr(request, "user", None)
                    actor_user = (
                        user if getattr(user, "is_authenticated", False) else None
                    )
                    history_entry = PdfProcessingHistory.objects.create(
                        pdf=pdf,
                        operation=PdfProcessingHistory.OPERATION_PDF_REDACTION,
                        source_type=source_type,
                        redaction_manifest=manifest,
                        note=note,
                        client_source_sha256=client_source_sha256,
                        source_sha256=source_sha256,
                        processed_file_name=str(
                            getattr(pdf.processed_file, "name", "")
                        ),
                        actor_user=actor_user,
                        actor_username=getattr(user, "username", "") if user else "",
                        actor_email=getattr(user, "email", "") if user else "",
                    )

                    pdf.refresh_from_db(fields=["date_modified", "state"])
                    status_after = _state_status_value(pdf.state) or status_before

                    record_operation(
                        request,
                        action=PdfProcessingHistory.OPERATION_PDF_REDACTION,
                        resource_type="pdf",
                        resource_id=pdf.pk,
                        status_before=status_before,
                        status_after=status_after,
                        meta={
                            "revision_id": history_entry.pk,
                            "source_type": source_type,
                            "client_source_sha256": client_source_sha256 or None,
                            "source_sha256": source_sha256,
                        },
                    )

                response_payload = PdfRedactionResponse(
                    file_id=pdf.pk,
                    revision_id=history_entry.pk,
                    processed_stream_url=(
                        f"/api/media/pdfs/{pdf.pk}/stream/?type=processed"
                    ),
                    status="done_processing_anonymization",
                    anonymization_validated=False,
                    updated_at=pdf.date_modified.isoformat(),
                )
                return Response(
                    response_payload.model_dump(mode="python"),
                    status=status.HTTP_201_CREATED,
                )

            except Exception:
                logger.exception(
                    "Unexpected failure applying redactions to report %s", pk
                )
                return Response(
                    {"error": "unexpected backend failure while applying redactions."},
                    status=status.HTTP_500_INTERNAL_SERVER_ERROR,
                )


class PdfProcessingHistoryView(APIView):
    """
    GET /api/media/pdfs/<pk>/processing-history/
    """

    permission_classes = [EnvironmentAwarePermission, PolicyPermission]

    def get(self, request, pk: int):
        pdf = RawPdfFile.objects.filter(pk=pk).first()
        if pdf is None:
            return Response(
                {"error": f"report with ID {pk} not found."},
                status=status.HTTP_404_NOT_FOUND,
            )

        history = PdfProcessingHistory.objects.filter(pdf_id=pdf.pk).order_by(
            "-created_at"
        )
        serializer = PdfProcessingHistorySerializer(
            history,
            many=True,
            context={"request": request},
        )
        return Response(serializer.data, status=status.HTTP_200_OK)



"""
Video Correction API Views

Provides endpoints for video analysis, masking, frame removal, and processing history.
Created as part of Phase 1.1: Video Correction API Endpoints.

Available Functions from lx_anonymizer (already implemented):
- FrameCleaner.analyze_video_sensitivity() - Frame analysis
- FrameCleaner.clean_video() - Complete anonymization
- FrameCleaner._mask_video() - FFmpeg masking
- FrameCleaner.remove_frames_from_video() - Frame removal
- VideoImportService._get_processor_roi_info() - ROI extraction
"""

import json
import logging
import os
from pathlib import Path
from typing import Any

from django.conf import settings
from django.shortcuts import get_object_or_404
from lx_anonymizer import FrameCleaner
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from endoreg_db.models import (
    LabelVideoSegment,
    VideoFile,
    VideoMetadata,
    VideoProcessingHistory,
)
from endoreg_db.serializers import VideoProcessingHistorySerializer
from endoreg_db.serializers.video.video_file_detail import VideoDetailSerializer
from endoreg_db.utils.paths import ANONYM_VIDEO_DIR
from endoreg_db.utils.permissions import EnvironmentAwarePermission

logger = logging.getLogger(__name__)


def update_processed_file(video, output_path: Path):
    try:
        rel_path = output_path.relative_to(ANONYM_VIDEO_DIR)
        assert video.processed_file is not None
    except ValueError:
        rel_path = output_path.relative_to(Path(settings.MEDIA_ROOT))

    video.processed_file.name = str(rel_path)
    video.save(update_fields=["processed_file"])


def _resolve_processing_method(payload: Any) -> str:
    explicit = payload.get("processing_method")
    if explicit:
        return str(explicit)

    use_streaming = payload.get("use_streaming")
    if use_streaming is None:
        return "streaming"
    return "streaming" if bool(use_streaming) else "direct"


def _masked_output_path(video: VideoFile) -> Path:
    return ANONYM_VIDEO_DIR / f"{video.video_hash}_masked.mp4"


def _cleaned_output_path(video: VideoFile) -> Path:
    return ANONYM_VIDEO_DIR / f"{video.video_hash}_cleaned.mp4"


def _part_output_path(output_path: Path) -> Path:
    return output_path.with_name(f"{output_path.stem}.part{output_path.suffix}")


def _promote_processed_output(temp_path: Path, final_path: Path) -> Path:
    if not temp_path.exists():
        raise FileNotFoundError(f"Temporary processed output missing: {temp_path}")
    if temp_path.stat().st_size <= 0:
        raise RuntimeError(f"Temporary processed output is empty: {temp_path}")
    final_path.parent.mkdir(parents=True, exist_ok=True)
    os.replace(temp_path, final_path)
    return final_path


def _normalize_custom_roi(roi: Any) -> dict[str, Any] | None:
    if not isinstance(roi, dict):
        return None
    if {"x", "y", "width", "height"}.issubset(roi.keys()):
        return roi
    if {"endoscope_x", "endoscope_y", "endoscope_width", "endoscope_height"}.issubset(
        roi.keys()
    ):
        return {
            "x": roi.get("endoscope_x"),
            "y": roi.get("endoscope_y"),
            "width": roi.get("endoscope_width"),
            "height": roi.get("endoscope_height"),
        }
    return roi


class VideoCorrectionView(APIView):
    """
    GET /api/video/media/video-correction/{id}/ - Get video details for correction
    """

    permission_classes = [EnvironmentAwarePermission]

    def get(self, request, pk):
        video = get_object_or_404(VideoFile, pk=pk)
        ser = VideoDetailSerializer(video, context={"request": request})
        return Response(ser.data, status=status.HTTP_200_OK)


def update_segments_after_frame_removal(video: VideoFile, removed_frames: list) -> dict:
    """
    Update LabelVideoSegment frame boundaries after frame removal.

    This function shifts segment start/end frames based on which frames were removed.
    Segments are deleted if all their frames are removed.

    Args:
        video: VideoFile instance whose segments should be updated
        removed_frames: List of frame numbers that were removed (sorted)

    Returns:
        dict: {
            'segments_updated': int,
            'segments_deleted': int,
            'segments_unchanged': int
        }

    Algorithm:
        For each segment:
        1. Count frames removed before segment → shift start_frame
        2. Count frames removed within segment → shift end_frame
        3. Delete segment if start_frame >= end_frame (all frames removed)

    Example:
        Original segment: frames 100-200
        Removed frames: [50, 75, 120, 150, 180]

        Frames before segment (< 100): 2 frames (50, 75)
        Frames within segment (100-200): 3 frames (120, 150, 180)

        New segment: frames (100-2) to (200-2-3) = 98-195
    """
    if not removed_frames:
        return {"segments_updated": 0, "segments_deleted": 0, "segments_unchanged": 0}

    removed_frames = sorted(set(removed_frames))  # Ensure sorted and unique
    segments = LabelVideoSegment.objects.filter(video_file=video).order_by(
        "start_frame_number"
    )

    segments_updated = 0
    segments_deleted = 0
    segments_unchanged = 0

    for segment in segments:
        original_start = segment.start_frame_number
        original_end = segment.end_frame_number

        # Count frames removed before this segment
        frames_before = sum(1 for f in removed_frames if f < original_start)

        # Count frames removed within this segment
        frames_within = sum(
            1 for f in removed_frames if original_start <= f <= original_end
        )

        # Calculate new boundaries
        new_start = original_start - frames_before
        new_end = original_end - frames_before - frames_within

        # Delete segment if all frames were removed
        if new_start >= new_end:
            logger.info(
                f"Deleting segment {segment.pk} (original: {original_start}-{original_end}) "
                f"- all {frames_within} frames removed"
            )
            segment.delete()
            segments_deleted += 1
        elif new_start != original_start or new_end != original_end:
            # Update segment boundaries
            logger.info(
                f"Updating segment {segment.pk}: "
                f"{original_start}-{original_end} → {new_start}-{new_end} "
                f"(before: {frames_before}, within: {frames_within})"
            )
            segment.start_frame_number = new_start
            segment.end_frame_number = new_end
            segment.save(update_fields=["start_frame_number", "end_frame_number"])
            segments_updated += 1
        else:
            # No change needed
            segments_unchanged += 1

    logger.info(
        f"Segment update complete for video {video.pk}: "
        f"{segments_updated} updated, {segments_deleted} deleted, {segments_unchanged} unchanged"
    )

    return {
        "segments_updated": segments_updated,
        "segments_deleted": segments_deleted,
        "segments_unchanged": segments_unchanged,
    }


class VideoProcessingHistoryView(APIView):
    """
    GET /api/media/videos/{pk}/processing-history/

    Retrieve processing history for a video.

    Returns list of processing operations:
        [
            {
                "id": 1,
                "video": 123,
                "operation": "mask_overlay",
                "operation_display": "Mask Overlay",
                "status": "success",
                "status_display": "Success",
                "config": {"mask_type": "device", "device_name": "olympus"},
                "output_file": "/path/to/masked_video.mp4",
                "download_url": "/api/media/processed-videos/123/1/",
                "details": "",
                "task_id": "abc-123-def",
                "created_at": "2025-10-09T10:00:00Z",
                "completed_at": "2025-10-09T10:05:00Z",
                "duration": 300.5,
                "is_complete": true
            },
            ...
        ]
    """

    def get(self, request, pk):
        """Get processing history for a video."""
        video = get_object_or_404(VideoFile, pk=pk)

        # Get all history records, newest first
        history = VideoProcessingHistory.objects.filter(video=video).order_by(
            "-created_at"
        )

        serializer = VideoProcessingHistorySerializer(
            history, many=True, context={"request": request}
        )
        return Response(serializer.data)


class VideoApplyMaskView(APIView):
    """
    POST /api/media/videos/{pk}/apply-mask/

    Apply device mask or custom ROI mask to video.

    Request body:
        {
            "mask_type": "device",           // or "custom"
            "device_name": "olympus",        // required if mask_type=device
            "roi": {                         // required if mask_type=custom
                "x": 550,
                "y": 0,
                "width": 1350,
                "height": 1080
            },
            "processing_method": "streaming" // or "direct" (default: streaming)
        }

    Returns:
        {
            "task_id": "abc-123-def",        // Celery task ID (future)
            "output_file": "/path/to/masked_video.mp4",
            "message": "Masking complete",
            "processing_time": 125.5
        }

    Note: Currently synchronous. Will be converted to Celery task in Phase 1.2.
    """

    def post(self, request, pk):
        """Apply masking to video."""
        video = get_object_or_404(VideoFile, pk=pk)

        # Extract parameters
        mask_type = request.data.get("mask_type", "device")
        device_name = request.data.get("device_name")
        roi = _normalize_custom_roi(
            request.data.get("roi") or request.data.get("custom_mask")
        )
        processing_method = _resolve_processing_method(request.data)

        # Validate required parameters
        if mask_type == "device" and not device_name:
            return Response(
                {"error": "device_name required for device mask"},
                status=status.HTTP_400_BAD_REQUEST,
            )

        if mask_type == "custom" and not roi:
            return Response(
                {"error": "roi required for custom mask"},
                status=status.HTTP_400_BAD_REQUEST,
            )

        # Create processing history record
        history = VideoProcessingHistory.objects.create(
            video=video,
            operation=VideoProcessingHistory.OPERATION_MASKING,
            status=VideoProcessingHistory.STATUS_PENDING,
            config={
                "mask_type": mask_type,
                "device_name": device_name,
                "roi": roi,
                "processing_method": processing_method,
            },
        )

        try:
            history.mark_running()

            # Initialize FrameCleaner
            frame_cleaner = FrameCleaner()

            # Get video paths
            raw_file_path = video.get_raw_file_path()
            if raw_file_path is None:
                raise FileNotFoundError(
                    f"Raw video file not found for correction: {video.video_hash}"
                )
            video_path = Path(raw_file_path)
            output_path = _masked_output_path(video)
            temp_output_path = _part_output_path(output_path)
            output_path.parent.mkdir(parents=True, exist_ok=True)
            temp_output_path.unlink(missing_ok=True)

            # Load or create mask config against current lx_anonymizer API.
            if mask_type == "device":
                frame_cleaner.mask_application.device_name = str(
                    device_name or "olympus_cv_1500"
                )
                mask_config = frame_cleaner.mask_application._load_mask()
            else:  # custom
                mask_config = (
                    frame_cleaner.mask_application.create_mask_config_from_roi(
                        endoscope_image_roi=roi or {},
                    )
                )

            # Apply mask using the current streaming masker API.
            import time

            start_time = time.time()

            success = frame_cleaner.mask_application.mask_video_streaming(
                input_video=video_path,
                mask_config=mask_config,
                output_video=temp_output_path,
                use_named_pipe=processing_method == "streaming",
            )

            processing_time = time.time() - start_time

            if success:
                # Update video record with anonymized file
                processed_file_path = _promote_processed_output(
                    temp_output_path, output_path
                )
                update_processed_file(video, processed_file_path)
                # Mark history as success
                history.mark_success(
                    output_file=str(output_path),
                    details=f"Masking completed in {processing_time:.1f}s",
                )

                logger.info(f"Video {pk} masked successfully: {output_path}")

                return Response(
                    {
                        "task_id": None,  # Will be Celery task ID in Phase 1.2
                        "output_file": str(output_path),
                        "message": "Masking complete",
                        "processing_time": processing_time,
                    }
                )
            else:
                raise Exception("Masking failed - check FFmpeg logs")

        except Exception as e:
            logger.error(f"Video masking failed for {pk}: {str(e)}", exc_info=True)
            temp_output_path = _part_output_path(_masked_output_path(video))
            temp_output_path.unlink(missing_ok=True)

            history.mark_failure(str(e))

            return Response(
                {"error": f"Masking failed: {str(e)}"},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )


class VideoRemoveFramesView(APIView):
    """
    POST /api/media/videos/{pk}/remove-frames/

    Remove specified frames from video.

    Request body:
        {
            "frame_list": [10, 15, 20, 30],      // explicit frame numbers
            // OR
            "frame_ranges": "10-20,30,45-50",    // range string
            // OR
            "detection_method": "automatic",     // use analysis results

            "processing_method": "streaming"     // or "traditional"
        }

    Returns:
        {
            "task_id": "abc-123-def",
            "output_file": "/path/to/cleaned_video.mp4",
            "frames_removed": 42,
            "message": "Frame removal complete",
            "processing_time": 180.3
        }

    Note: Currently synchronous. Will be converted to Celery task in Phase 1.2.
    """

    def post(self, request, pk):
        """Remove frames from video."""
        video = get_object_or_404(VideoFile, pk=pk)

        # Extract parameters
        frame_list = request.data.get("frame_list") or request.data.get("manual_frames")
        frame_ranges = request.data.get("frame_ranges")
        detection_method = request.data.get("detection_method")
        selection_method = request.data.get("selection_method")
        if detection_method is None and selection_method == "automatic":
            detection_method = "automatic"
        processing_method = _resolve_processing_method(request.data)

        # Determine frames to remove
        frames_to_remove = []

        if frame_list:
            frames_to_remove = frame_list
        elif frame_ranges:
            frames_to_remove = self._parse_frame_ranges(frame_ranges)
        elif detection_method == "automatic":
            # Use existing analysis results
            try:
                metadata = VideoMetadata.objects.get(video=video)
                if metadata.sensitive_frame_ids:
                    frames_to_remove = json.loads(metadata.sensitive_frame_ids)
                else:
                    return Response(
                        {"error": "No analysis results available. Run analysis first."},
                        status=status.HTTP_400_BAD_REQUEST,
                    )
            except VideoMetadata.DoesNotExist:
                return Response(
                    {"error": "Video not analyzed. Run analysis first."},
                    status=status.HTTP_400_BAD_REQUEST,
                )
        else:
            return Response(
                {
                    "error": "Must provide frame_list, frame_ranges, or detection_method=automatic"
                },
                status=status.HTTP_400_BAD_REQUEST,
            )

        if not frames_to_remove:
            return Response(
                {"error": "No frames to remove"}, status=status.HTTP_400_BAD_REQUEST
            )

        # Create processing history record
        history = VideoProcessingHistory.objects.create(
            video=video,
            operation=VideoProcessingHistory.OPERATION_FRAME_REMOVAL,
            status=VideoProcessingHistory.STATUS_PENDING,
            config={
                "frames_to_remove": frames_to_remove,
                "frame_count": len(frames_to_remove),
                "processing_method": processing_method,
            },
        )

        try:
            history.mark_running()

            # Initialize FrameCleaner
            frame_cleaner = FrameCleaner()

            # Get video paths
            raw_file_path = video.get_raw_file_path()
            if raw_file_path is None:
                raise FileNotFoundError(
                    f"Raw video file not found for frame removal: {video.video_hash}"
                )
            video_path = Path(raw_file_path)
            output_path = _cleaned_output_path(video)
            temp_output_path = _part_output_path(output_path)
            output_path.parent.mkdir(parents=True, exist_ok=True)
            temp_output_path.unlink(missing_ok=True)

            # Remove frames using the current streaming removal API.
            import time

            start_time = time.time()

            success = frame_cleaner.remove_frames_from_video_streaming(
                original_video=video_path,
                frames_to_remove=frames_to_remove,
                output_video=temp_output_path,
                total_frames=video.frame_count,
                use_named_pipe=processing_method == "streaming",
            )

            processing_time = time.time() - start_time

            if success:
                # Update video record
                final_output_path = _promote_processed_output(
                    temp_output_path, output_path
                )
                update_processed_file(video, final_output_path)

                # Phase 1.4: Update LabelVideoSegments (shift frame numbers)
                segment_update_result = update_segments_after_frame_removal(
                    video, frames_to_remove
                )

                # Mark history as success with segment update info
                details_parts = [
                    f"Removed {len(frames_to_remove)} frames in {processing_time:.1f}s"
                ]
                if segment_update_result["segments_updated"] > 0:
                    details_parts.append(
                        f"Updated {segment_update_result['segments_updated']} segments"
                    )
                if segment_update_result["segments_deleted"] > 0:
                    details_parts.append(
                        f"Deleted {segment_update_result['segments_deleted']} segments (all frames removed)"
                    )

                history.mark_success(
                    output_file=str(final_output_path), details="; ".join(details_parts)
                )

                logger.info(
                    f"Video {pk} cleaned: removed {len(frames_to_remove)} frames"
                )

                return Response(
                    {
                        "task_id": None,  # Will be Celery task ID in Phase 1.2
                        "output_file": str(final_output_path),
                        "frames_removed": len(frames_to_remove),
                        "segment_updates": segment_update_result,
                        "message": "Frame removal complete",
                        "processing_time": processing_time,
                    }
                )
            else:
                raise Exception("Frame removal failed - check FFmpeg logs")

        except Exception as e:
            logger.error(f"Frame removal failed for {pk}: {str(e)}", exc_info=True)
            temp_output_path = _part_output_path(_cleaned_output_path(video))
            temp_output_path.unlink(missing_ok=True)

            history.mark_failure(str(e))

            return Response(
                {"error": f"Frame removal failed: {str(e)}"},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )

    def _parse_frame_ranges(self, ranges_str: str) -> list[int]:
        """
        Parse frame ranges string to list of frame numbers.

        Example: "10-20,30,45-50" -> [10,11,...,20,30,45,...,50]
        """
        frames: list[int] = []
        for part in ranges_str.split(","):
            part = part.strip()
            if "-" in part:
                start, end = map(int, part.split("-"))
                frames.extend(range(start, end + 1))
            else:
                frames.append(int(part))
        return sorted(set(frames))  # Remove duplicates and sort
